@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,7 +20,7 @@
  */
 /*
   File   : canonmn.cpp
-  Version: $Rev: 1110 $
+  Version: $Rev: 1366 $
   Authors: Andreas Huggel (ahu) <ahuggel@gmx.net>
            David Cannings (dc) <david@edeca.net>
   Credits: EXIF MakerNote of Canon by David Burren <http://www.burren.cx/david/canon.html>
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: canonmn.cpp 1110 2007-05-29 15:20:45Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: canonmn.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -890,7 +890,7 @@ namespace Exiv2 {
                                   ByteOrder byteOrder) const
     {
         DataBuf buf(1024);
-        memset(buf.pData_, 0x0, 1024);
+        std::memset(buf.pData_, 0x0, 1024);
         uint16_t len = 0;
         Entries::const_iterator end = entries_.end();
         for (Entries::const_iterator i = entries_.begin(); i != end; ++i) {
@@ -898,7 +898,7 @@ namespace Exiv2 {
                 uint16_t pos = i->tag() * 2;
                 uint16_t size = pos + static_cast<uint16_t>(i->size());
                 assert(size <= 1024);
-                memcpy(buf.pData_ + pos, i->data(), i->size());
+                std::memcpy(buf.pData_ + pos, i->data(), i->size());
                 if (len < size) len = size;
             }
         }
