@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2006-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      cr2image.cpp
-  Version:   $Rev: 1047 $
+  Version:   $Rev: 1366 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   22-Apr-06, ahu: created
 
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: cr2image.cpp 1047 2007-02-07 13:56:29Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: cr2image.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -69,6 +69,7 @@ namespace Exiv2 {
         { "*",         0x011a, Group::ifd0,    0 },
         { "*",         0x011b, Group::ifd0,    0 },
         { "*",         0x0128, Group::ifd0,    0 },
+        { "*",         0x02bc, Group::ifd0,    &TiffMetadataDecoder::decodeXmp  },
         { "*",         0x83bb, Group::ifd0,    &TiffMetadataDecoder::decodeIptc },
         { "*",         0x8649, Group::ifd0,    &TiffMetadataDecoder::decodeIptc }
     };
@@ -125,8 +126,10 @@ namespace Exiv2 {
             throw Error(3, "CR2");
         }
         clearMetadata();
+        Cr2Header cr2Header;
         TiffParser::decode(this, io_->mmap(), io_->size(),
-                           TiffCreator::create, Cr2Decoder::findDecoder);
+                           TiffCreator::create, Cr2Decoder::findDecoder,
+                           &cr2Header);
 
     } // Cr2Image::readMetadata
 
@@ -136,44 +139,43 @@ namespace Exiv2 {
         throw(Error(31, "CR2"));
     } // Cr2Image::writeMetadata
 
-    const uint16_t Cr2Header::tag_ = 42;
     const char* Cr2Header::cr2sig_ = "CR\2\0";
 
+    Cr2Header::Cr2Header()
+        : TiffHeaderBase(42, 16, littleEndian, 0x00000010),
+          offset2_(0x00000000)
+    {
+    }
+
+    Cr2Header::~Cr2Header()
+    {
+    }
+
     bool Cr2Header::read(const byte* pData, uint32_t size)
     {
         if (size < 16) return false;
 
         if (pData[0] == 0x49 && pData[1] == 0x49) {
-            byteOrder_ = littleEndian;
+            setByteOrder(littleEndian);
         }
         else if (pData[0] == 0x4d && pData[1] == 0x4d) {
-            byteOrder_ = bigEndian;
+            setByteOrder(bigEndian);
         }
         else {
             return false;
         }
-        if (tag_ != getUShort(pData + 2, byteOrder_)) return false;
-        offset_ = getULong(pData + 4, byteOrder_);
+        if (tag() != getUShort(pData + 2, byteOrder())) return false;
+        setOffset(getULong(pData + 4, byteOrder()));
         if (0 != memcmp(pData + 8, cr2sig_, 4)) return false;
-        offset2_ = getULong(pData + 12, byteOrder_);
+        offset2_ = getULong(pData + 12, byteOrder());
 
         return true;
     } // Cr2Header::read
 
-    void Cr2Header::print(std::ostream& os, const std::string& prefix) const
+    void Cr2Header::write(Blob& blob) const
     {
-        os << prefix
-           << _("Header, offset") << " = 0x" << std::setw(8) << std::setfill('0')
-           << std::hex << std::right << offset_;
-
-        switch (byteOrder_) {
-        case littleEndian:     os << ", " << _("little endian encoded"); break;
-        case bigEndian:        os << ", " << _("big endian encoded"); break;
-        case invalidByteOrder: break;
-        }
-        os << "\n";
-
-    } // Cr2Header::print
+        // Todo: Implement me!
+    }
 
     // *************************************************************************
     // free functions
