@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2005-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      crwimage.cpp
-  Version:   $Rev: 1079 $
+  Version:   $Rev: 1366 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   28-Aug-05, ahu: created
 
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: crwimage.cpp 1079 2007-03-13 13:20:55Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: crwimage.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
 
 // Define DEBUG to output debug information to std::cerr, e.g, by calling make
 // like this: make DEFS=-DDEBUG crwimage.o
@@ -1133,15 +1133,15 @@ namespace Exiv2 {
             uint32_t size = static_cast<uint32_t>(comment.size());
             if (cc && cc->size() > size) size = cc->size();
             DataBuf buf(size);
-            memset(buf.pData_, 0x0, buf.size_);
-            memcpy(buf.pData_, comment.data(), comment.size());
+            std::memset(buf.pData_, 0x0, buf.size_);
+            std::memcpy(buf.pData_, comment.data(), comment.size());
             pHead->add(pCrwMapping->crwTagId_, pCrwMapping->crwDir_, buf);
         }
         else {
             if (cc) {
                 // Just delete the value, do not remove the tag
                 DataBuf buf(cc->size());
-                memset(buf.pData_, 0x0, buf.size_);
+                std::memset(buf.pData_, 0x0, buf.size_);
                 cc->setValue(buf);
             }
         }
@@ -1216,13 +1216,13 @@ namespace Exiv2 {
         const ExifData::const_iterator ed = image.exifData().findKey(key);
         if (ed != image.exifData().end()) {
             struct tm tm;
-            memset(&tm, 0x0, sizeof(tm));
+            std::memset(&tm, 0x0, sizeof(tm));
             int rc = exifTime(ed->toString().c_str(), &tm);
             if (rc == 0) t = timegm(&tm);
         }
         if (t != 0) {
             DataBuf buf(12);
-            memset(buf.pData_, 0x0, 12);
+            std::memset(buf.pData_, 0x0, 12);
             ul2Data(buf.pData_, static_cast<uint32_t>(t), pHead->byteOrder());
             pHead->add(pCrwMapping->crwTagId_, pCrwMapping->crwDir_, buf);
         }
@@ -1252,8 +1252,8 @@ namespace Exiv2 {
             uint32_t size = 28;
             if (cc && cc->size() > size) size = cc->size();
             DataBuf buf(size);
-            memset(buf.pData_, 0x0, buf.size_);
-            if (cc) memcpy(buf.pData_ + 8, cc->pData() + 8, cc->size() - 8);
+            std::memset(buf.pData_, 0x0, buf.size_);
+            if (cc) std::memcpy(buf.pData_ + 8, cc->pData() + 8, cc->size() - 8);
             if (edX != edEnd && edX->size() == 4) {
                 edX->copy(buf.pData_, pHead->byteOrder());
             }
@@ -1325,7 +1325,7 @@ namespace Exiv2 {
     {
         const uint16_t size = 1024;
         DataBuf buf(size);
-        memset(buf.pData_, 0x0, buf.size_);
+        std::memset(buf.pData_, 0x0, buf.size_);
 
         uint16_t len = 0;
         const ExifData::const_iterator b = exifData.begin();
