@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -23,7 +23,7 @@
   @brief   Class CrwImage to access Canon Crw images.<BR>
            References:<BR>
            <a href="http://www.sno.phy.queensu.ca/~phil/exiftool/canon_raw.html">The Canon RAW (CRW) File Format</a> by Phil Harvey
-  @version $Rev: 705 $
+  @version $Rev: 1366 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    28-Aug-05, ahu: created
@@ -36,8 +36,6 @@
 #include "types.hpp"
 #include "image.hpp"
 #include "basicio.hpp"
-#include "exif.hpp"
-#include "iptc.hpp"
 
 // + standard includes
 #include <iosfwd>
@@ -51,6 +49,8 @@ namespace Exiv2 {
 
 // *****************************************************************************
 // class declarations
+    class ExifData;
+    class IptcData;
     class CiffHeader;
     class CiffComponent;
     struct CrwMapping;
@@ -82,25 +82,15 @@ namespace Exiv2 {
     }
 
     /*!
-      @brief Class to access raw Canon Crw images. Only Exif metadata and a
-             comment are supported. Crw format does not contain Iptc metadata.
+      @brief Class to access raw Canon CRW images. Only Exif metadata and a
+             comment are supported. CRW format does not contain IPTC metadata.
      */
     class CrwImage : public Image {
-        friend bool isCrwType(BasicIo& iIo, bool advance);
-
-        //! @name NOT Implemented
-        //@{
-        //! Copy constructor
-        CrwImage(const CrwImage& rhs);
-        //! Assignment operator
-        CrwImage& operator=(const CrwImage& rhs);
-        //@}
-
     public:
         //! @name Creators
         //@{
         /*!
-          @brief Constructor that can either open an existing Crw image or create
+          @brief Constructor that can either open an existing CRW image or create
               a new image from scratch. If a new image is to be created, any
               existing data is overwritten. Since the constructor can not return
               a result, callers should check the good() method after object
@@ -115,81 +105,33 @@ namespace Exiv2 {
               or if a new file should be created (true).
          */
         CrwImage(BasicIo::AutoPtr io, bool create);
-        //! Destructor
-        ~CrwImage() {}
         //@}
 
         //! @name Manipulators
         //@{
-        void            readMetadata();
+        void readMetadata();
+        void writeMetadata();
         /*!
-          @brief Todo: Write metadata back to the image. This method is not
-                 yet implemented.
+          @brief Not supported. CRW format does not contain IPTC metadata.
+              Calling this function will throw an Error(32).
          */
-        void            writeMetadata();
-        void            setExifData(const ExifData& exifData);
-        void            clearExifData();
-        /*!
-          @brief Not supported. Crw format does not contain Iptc metadata.
-                 Calling this function will do nothing.
-         */
-        void            setIptcData(const IptcData& iptcData);
-        /*!
-          @brief Not supported. Crw format does not contain Iptc metadata.
-                 Calling this function will do nothing.
-         */
-        void            clearIptcData();
-        void            setComment(const std::string& comment);
-        void            clearComment();
-        void            setMetadata(const Image& image);
-        void            clearMetadata();
-        ExifData&       exifData()       { return exifData_; }
-        IptcData&       iptcData()       { return iptcData_; }
+        void setIptcData(const IptcData& iptcData);
         //@}
 
         //! @name Accessors
         //@{
-        bool            good()     const;
-        const ExifData& exifData() const { return exifData_; }
-        const IptcData& iptcData() const { return iptcData_; }
-        std::string     comment()  const { return comment_; }
-        BasicIo&        io()       const { return *io_; }
+        std::string mimeType() const { return "image/x-canon-crw"; }
         //@}
 
     private:
-        //! @name Accessors
+        //! @name NOT Implemented
         //@{
-        /*!
-          @brief Determine if the content of the BasicIo instance is a Crw image.
-
-          The advance flag determines if the read position in the stream is
-          moved (see below). This applies only if the type matches and the
-          function returns true. If the type does not match, the stream
-          position is not changed. However, if reading from the stream fails,
-          the stream position is undefined. Consult the stream state to obtain
-          more information in this case.
-
-          @param iIo BasicIo instance to read from.
-          @param advance Flag indicating whether the position of the io
-              should be advanced by the number of characters read to
-              analyse the data (true) or left at its original
-              position (false). This applies only if the type matches.
-          @return  true  if the data matches the type of this class;<BR>
-                   false if the data does not match
-         */
-        bool isThisType(BasicIo& iIo, bool advance) const;
-        /*!
-          @brief Todo: Write Crw header. Not implemented yet.
-         */
-        int writeHeader(BasicIo& oIo) const;
+        //! Copy constructor
+        CrwImage(const CrwImage& rhs);
+        //! Assignment operator
+        CrwImage& operator=(const CrwImage& rhs);
         //@}
 
-        // DATA
-        BasicIo::AutoPtr  io_;                  //!< Image data io pointer
-        ExifData          exifData_;            //!< Exif data container
-        IptcData          iptcData_;            //!< Iptc data container
-        std::string       comment_;             //!< User comment
-
     }; // class CrwImage
 
     /*!
