@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    exiv2.hpp
   @brief   Defines class Params, used for the command line handling of exiv2
-  @version $Rev: 1081 $
+  @version $Rev: 1366 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    08-Dec-03, ahu: created
@@ -43,9 +43,9 @@
 // class definitions
 
 //! Command identifiers
-enum CmdId { invalidCmdId, add, set, del };
+enum CmdId { invalidCmdId, add, set, del, reg };
 //! Metadata identifiers
-enum MetadataId { invalidMetadataId, iptc, exif };
+enum MetadataId { invalidMetadataId, iptc, exif, xmp };
 //! Structure for one parsed modification command
 struct ModifyCmd {
     //! C'tor
@@ -123,7 +123,7 @@ public:
     void cleanup();
 
     //! Enumerates print modes
-    enum PrintMode { pmSummary, pmList, pmIptc, pmComment };
+    enum PrintMode { pmSummary, pmList, pmIptc, pmXmp, pmComment };
 
     //! Individual items to print
     enum PrintItem {
@@ -141,10 +141,20 @@ public:
     };
 
     //! Enumerates common targets, bitmap
-    enum CommonTarget { ctExif = 1, ctIptc = 2, ctComment = 4, ctThumb = 8 };
+    enum CommonTarget { ctExif = 1, ctIptc = 2, ctComment = 4, ctThumb = 8, ctXmp = 16, ctXmpPacket = 32 };
     //! Enumerates the policies to handle existing files in rename action
     enum FileExistsPolicy { overwritePolicy, renamePolicy, askPolicy };
 
+    //! Enumerates year, month and day adjustments.
+    enum Yod { yodYear, yodMonth, yodDay };
+
+    //! Structure for year, month and day adjustment command line arguments.
+    struct YodAdjust {
+        bool        flag_;              //!< Adjustment flag.
+        const char* option_;            //!< Adjustment option string.
+        long        adjustment_;        //!< Adjustment value.
+    };
+
     bool help_;                         //!< Help option flag.
     bool version_;                      //!< Version option flag.
     bool verbose_;                      //!< Verbose (talkative) option flag.
@@ -163,6 +173,7 @@ public:
     int  target_;                       //!< What common target to process.
 
     long adjustment_;                   //!< Adjustment in seconds.
+    YodAdjust yodAdjust_[3];            //!< Year, month and day adjustment info.
     std::string format_;                //!< Filename format (-r option arg).
     bool formatSet_;                    //!< Whether the format is set with -r
     CmdFiles cmdFiles_;                 //!< Names of the modification command files
@@ -173,12 +184,20 @@ public:
     std::string suffix_;                //!< File extension of the file to insert
     Files files_;                       //!< List of non-option arguments.
 
+private:
+    //! Pointer to the global Params object.
+    static Params* instance_;
+    //! Initializer for year, month and day adjustment info.
+    static const YodAdjust emptyYodAdjust_[];
+
+    bool first_;
+
 private:
     /*!
       @brief Default constructor. Note that optstring_ is initialized here.
              The c'tor is private to force instantiation through instance().
      */
-    Params() : optstring_(":hVvfbuktTFa:r:p:P:d:e:i:c:m:M:l:S:"),
+    Params() : optstring_(":hVvfbuktTFa:Y:O:D:r:p:P:d:e:i:c:m:M:l:S:"),
                help_(false),
                version_(false),
                verbose_(false),
@@ -193,11 +212,16 @@ private:
                printMode_(pmSummary),
                printItems_(0),
                action_(0),
-               target_(ctExif|ctIptc|ctComment),
+               target_(ctExif|ctIptc|ctComment|ctXmp),
                adjustment_(0),
                format_("%Y%m%d_%H%M%S"),
                formatSet_(false),
-               first_(true) {}
+               first_(true)
+    {
+        yodAdjust_[yodYear]  = emptyYodAdjust_[yodYear];
+        yodAdjust_[yodMonth] = emptyYodAdjust_[yodMonth];
+        yodAdjust_[yodDay]   = emptyYodAdjust_[yodDay];
+    }
 
     //! Prevent copy-construction: not implemented.
     Params(const Params& rhs);
@@ -206,6 +230,7 @@ private:
     //@{
     int evalRename(int opt, const std::string& optarg);
     int evalAdjust(const std::string& optarg);
+    int evalYodAdjust(const Yod& yod, const std::string& optarg);
     int evalPrint(const std::string& optarg);
     int evalPrintCols(const std::string& optarg);
     int evalDelete(const std::string& optarg);
@@ -214,11 +239,6 @@ private:
     int evalModify(int opt, const std::string& optarg);
     //@}
 
-    //! Pointer to the global Params object.
-    static Params* instance_;
-
-    bool first_;
-
 public:
     /*!
       @brief Call Getopt::getopt() with optstring, to inititate command line
