@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,7 +20,7 @@
  */
 /*
   File:      image.cpp
-  Version:   $Rev: 1046 $
+  Version:   $Rev: 1366 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Brad Schick (brad) <brad@robotbattle.com>
   History:   26-Jan-04, ahu: created
@@ -30,7 +30,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: image.cpp 1046 2007-02-06 15:00:23Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: image.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -53,6 +53,7 @@ EXIV2_RCSID("@(#) $Id: image.cpp 1046 2007-02-06 15:00:23Z ahuggel $")
 #endif // EXV_HAVE_LIBZ
 #include "rafimage.hpp"
 #include "tiffimage.hpp"
+#include "orfimage.hpp"
 
 // + standard includes
 #include <cerrno>
@@ -73,20 +74,21 @@ EXIV2_RCSID("@(#) $Id: image.cpp 1046 2007-02-06 15:00:23Z ahuggel $")
 namespace Exiv2 {
 
     const ImageFactory::Registry ImageFactory::registry_[] = {
-        //image type       creation fct     type check  Exif mode    IPTC mode    Comment mode
-        //---------------  ---------------  ----------  -----------  -----------  ------------
-        { ImageType::jpeg, newJpegInstance, isJpegType, amReadWrite, amReadWrite, amReadWrite },
-        { ImageType::exv,  newExvInstance,  isExvType,  amReadWrite, amReadWrite, amReadWrite },
-        { ImageType::cr2,  newCr2Instance,  isCr2Type,  amRead,      amRead,      amNone      },
-        { ImageType::crw,  newCrwInstance,  isCrwType,  amReadWrite, amNone,      amReadWrite },
-        { ImageType::mrw,  newMrwInstance,  isMrwType,  amRead,      amRead,      amNone      },
-        { ImageType::tiff, newTiffInstance, isTiffType, amRead,      amRead,      amNone      },
-#ifdef EXV_HAVE_LIBZ
-        { ImageType::png,  newPngInstance,  isPngType,  amRead,      amRead,      amNone      },
-#endif // EXV_HAVE_LIBZ
-        { ImageType::raf,  newRafInstance,  isRafType,  amRead,      amRead,      amNone      },
-        // End of list marker
-        { ImageType::none, 0,               0,          amNone,      amNone,      amNone      }
+        //image type       creation fct     type check  Exif mode    IPTC mode    XMP mode     Comment mode
+        //---------------  ---------------  ----------  -----------  -----------  -----------  ------------
+        { ImageType::jpeg, newJpegInstance, isJpegType, amReadWrite, amReadWrite, amReadWrite, amReadWrite },
+        { ImageType::exv,  newExvInstance,  isExvType,  amReadWrite, amReadWrite, amReadWrite, amReadWrite },
+        { ImageType::cr2,  newCr2Instance,  isCr2Type,  amRead,      amRead,      amRead,      amNone      },
+        { ImageType::crw,  newCrwInstance,  isCrwType,  amReadWrite, amNone,      amNone,      amReadWrite },
+        { ImageType::mrw,  newMrwInstance,  isMrwType,  amRead,      amRead,      amRead,      amNone      },
+        { ImageType::tiff, newTiffInstance, isTiffType, amRead,      amRead,      amRead,      amNone      },
+        { ImageType::orf,  newOrfInstance,  isOrfType,  amRead,      amRead,      amRead,      amNone      },
+#ifdef EXV_HAVE_LIBZ                                                                           
+        { ImageType::png,  newPngInstance,  isPngType,  amRead,      amRead,      amRead,      amNone      },
+#endif // EXV_HAVE_LIBZ                                                                        
+        { ImageType::raf,  newRafInstance,  isRafType,  amRead,      amRead,      amRead,      amNone      },
+        // End of list marker                                                                  
+        { ImageType::none, 0,               0,          amNone,      amNone,      amNone,      amNone      }
     };
 
     bool ImageFactory::Registry::operator==(const int& imageType) const
@@ -99,7 +101,12 @@ namespace Exiv2 {
                  BasicIo::AutoPtr io)
         : io_(io),
           imageType_(imageType),
-          supportedMetadata_(supportedMetadata)
+          supportedMetadata_(supportedMetadata),
+#ifdef EXV_HAVE_XMP_TOOLKIT
+          writeXmpFromPacket_(false)
+#else 
+          writeXmpFromPacket_(true)
+#endif
     {
     }
 
@@ -107,6 +114,8 @@ namespace Exiv2 {
     {
         clearExifData();
         clearIptcData();
+        clearXmpPacket();
+        clearXmpData();
         clearComment();
     }
 
@@ -114,6 +123,8 @@ namespace Exiv2 {
     {
         setExifData(image.exifData());
         setIptcData(image.iptcData());
+        setXmpPacket(image.xmpPacket());
+        setXmpData(image.xmpData());
         setComment(image.comment());
     }
 
@@ -137,6 +148,37 @@ namespace Exiv2 {
         iptcData_ = iptcData;
     }
 
+    void Image::clearXmpPacket()
+    {
+        xmpPacket_.clear();
+        writeXmpFromPacket(true);
+    }
+
+    void Image::setXmpPacket(const std::string& xmpPacket)
+    {
+        xmpPacket_ = xmpPacket;
+        writeXmpFromPacket(true);
+    }
+
+    void Image::clearXmpData()
+    {
+        xmpData_.clear();
+        writeXmpFromPacket(false);
+    }
+
+    void Image::setXmpData(const XmpData& xmpData)
+    {
+        xmpData_ = xmpData;
+        writeXmpFromPacket(false);
+    }
+
+    void Image::writeXmpFromPacket(bool flag)
+    {
+#ifdef EXV_HAVE_XMP_TOOLKIT
+        writeXmpFromPacket_ = flag; 
+#endif
+    }
+
     void Image::clearComment()
     {
         comment_.erase();
@@ -176,6 +218,9 @@ namespace Exiv2 {
         case mdIptc:
             am = r->iptcSupport_;
             break;
+        case mdXmp:
+            am = r->xmpSupport_;
+            break;
         case mdComment:
             am = r->commentSupport_;
             break;
