@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      iptc.cpp
-  Version:   $Rev: 1039 $
+  Version:   $Rev: 1366 $
   Author(s): Brad Schick (brad) <brad@robotbattle.com>
   History:   31-July-04, brad: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: iptc.cpp 1039 2007-02-02 10:09:31Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: iptc.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -133,6 +133,9 @@ namespace Exiv2 {
 
     int IptcData::load(const byte* buf, long len)
     {
+#ifdef DEBUG
+        std::cerr << "IptcData::load, len = " << len << "\n";
+#endif
         const byte* pRead = buf;
         iptcMetadata_.clear();
 
@@ -166,7 +169,15 @@ namespace Exiv2 {
                 pRead += 2;
             }
             if (pRead + sizeData <= buf + len) {
-                readData(dataSet, record, pRead, sizeData);
+                int rc = 0;
+                if ((rc = readData(dataSet, record, pRead, sizeData)) != 0) {
+#ifndef SUPPRESS_WARNINGS
+                    std::cerr << "Warning: "
+                              << "Failed to read IPTC dataset "
+                              << IptcKey(dataSet, record)
+                              << " (rc = " << rc << "); skipped.\n";
+#endif
+                }
             }
 #ifndef SUPPRESS_WARNINGS
             else {
@@ -195,7 +206,7 @@ namespace Exiv2 {
         else if (1 == rc) {
             // If the first attempt failed, try with a string value
             value = Value::create(string);
-            int rc = value->read(data, sizeData, bigEndian);
+            rc = value->read(data, sizeData, bigEndian);
             if (0 == rc) {
                 IptcKey key(dataSet, record);
                 add(key, value.get());
