@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,7 +20,7 @@
  */
 /*
   File:      jpgimage.cpp
-  Version:   $Rev: 1126 $
+  Version:   $Rev: 1366 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Brad Schick (brad) <brad@robotbattle.com>
   History:   15-Jan-05, brad: split out from image.cpp
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: jpgimage.cpp 1126 2007-06-14 13:49:55Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: jpgimage.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -59,6 +59,7 @@ namespace Exiv2 {
     const byte     JpegBase::com_      = 0xfe;
     const char     JpegBase::exifId_[] = "Exif\0\0";
     const char     JpegBase::jfifId_[] = "JFIF\0";
+    const char     JpegBase::xmpId_[]  = "http://ns.adobe.com/xap/1.0/\0";
 
     const char     Photoshop::ps3Id_[] = "Photoshop 3.0\0";
     const char     Photoshop::bimId_[] = "8BIM";
@@ -79,7 +80,9 @@ namespace Exiv2 {
         assert(sizeData);
         // Used for error checking
         long position = 0;
-
+#ifdef DEBUG
+        std::cerr << "Photoshop::locateIrb: ";
+#endif
         // Data should follow Photoshop format, if not exit
         while (   position <= sizePsData - 14
                && memcmp(pPsData + position, Photoshop::bimId_, 4) == 0) {
@@ -87,26 +90,52 @@ namespace Exiv2 {
             position += 4;
             uint16_t type = getUShort(pPsData + position, bigEndian);
             position += 2;
-
+#ifdef DEBUG
+            std::cerr << "0x" << std::hex << type << std::dec << " ";
+#endif
             // Pascal string is padded to have an even size (including size byte)
             byte psSize = pPsData[position] + 1;
             psSize += (psSize & 1);
             position += psSize;
-            if (position >= sizePsData) return -2;
-
-            // Data is also padded to be even
+            if (position + 4 > sizePsData) {
+#ifndef SUPPRESS_WARNINGS
+                std::cerr << "Error: "
+                          << "Invalid Photoshop IRB\n";
+#endif
+                return -2;
+            }
             uint32_t dataSize = getULong(pPsData + position, bigEndian);
             position += 4;
-            if (dataSize > static_cast<uint32_t>(sizePsData - position)) return -2;
-
+            if (dataSize > static_cast<uint32_t>(sizePsData - position)) {
+#ifndef SUPPRESS_WARNINGS
+                std::cerr << "Error: "
+                          << "Invalid Photoshop IRB data size "
+                          << dataSize << "\n";
+#endif
+                return -2;
+            }
+#ifndef DEBUG
+            if (   (dataSize & 1) 
+                && position + dataSize == static_cast<uint32_t>(sizePsData)) {
+                std::cerr << "Warning: "
+                          << "Photoshop IRB data is not padded to even size\n";
+            }
+#endif
             if (type == psTag) {
+#ifdef DEBUG
+                std::cerr << "ok\n";
+#endif
                 *sizeData = dataSize;
                 *sizeHdr = psSize + 10;
                 *record = hrd;
                 return 0;
             }
+            // Data size is also padded to be even
             position += dataSize + (dataSize & 1);
         }
+#ifdef DEBUG
+        std::cerr << "pPsData doesn't start with '8BIM'\n";
+#endif
         return 3;
     } // Photoshop::locateIrb
 
@@ -133,16 +162,14 @@ namespace Exiv2 {
         const byte* record    = pPsData;
         uint32_t    sizeIptc  = 0;
         uint32_t    sizeHdr   = 0;
+        DataBuf rc;
         // Safe to call with zero psData.size_
-        Photoshop::locateIptcIrb(pPsData, sizePsData,
-                                 &record, &sizeHdr, &sizeIptc);
-
+        if (0 > Photoshop::locateIptcIrb(pPsData, sizePsData,
+                                         &record, &sizeHdr, &sizeIptc)) {
+            return rc;
+        }
         Blob psBlob;
-        // Data is rounded to be even
-        const uint32_t sizeOldData = sizeHdr + sizeIptc + (sizeIptc & 1);
         const uint32_t sizeFront = static_cast<uint32_t>(record - pPsData);
-        const uint32_t sizeEnd = sizePsData - sizeFront - sizeOldData;
-
         // Write data before old record.
         if (sizePsData > 0 && sizeFront > 0) {
             append(psBlob, pPsData, sizeFront);
@@ -151,7 +178,7 @@ namespace Exiv2 {
         DataBuf rawIptc(iptcData.copy());
         if (rawIptc.size_ > 0) {
             byte tmpBuf[12];
-            memcpy(tmpBuf, Photoshop::bimId_, 4);
+            std::memcpy(tmpBuf, Photoshop::bimId_, 4);
             us2Data(tmpBuf + 4, iptc_, bigEndian);
             tmpBuf[6] = 0;
             tmpBuf[7] = 0;
@@ -161,11 +188,15 @@ namespace Exiv2 {
             // Data is padded to be even (but not included in size)
             if (rawIptc.size_ & 1) psBlob.push_back(0x00);
         }
-        // Write existing stuff after record
-        if (sizePsData > 0 && sizeEnd > 0) {
-            append(psBlob, record + sizeOldData, sizeEnd);
+        // Write existing stuff after record, data is rounded to be even.
+        const uint32_t sizeOldData = sizeHdr + sizeIptc + (sizeIptc & 1);
+        // Note: Because of the rounding, sizeFront + sizeOldData can be 
+        // _greater_ than sizePsData by 1 (not just equal), if the original
+        // data was not padded.
+        if (static_cast<uint32_t>(sizePsData) > sizeFront + sizeOldData) {
+            append(psBlob, record + sizeOldData,
+                   sizePsData - sizeFront - sizeOldData);
         }
-        DataBuf rc;
         if (psBlob.size() > 0) rc = DataBuf(&psBlob[0], static_cast<long>(psBlob.size()));
 #ifdef DEBUG
         std::cerr << "IRB block at the end of Photoshop::setIptcIrb\n";
@@ -224,10 +255,12 @@ namespace Exiv2 {
             throw Error(15);
         }
         clearMetadata();
-        int search = 3;
-        const long bufMinSize = 16;
+        int search = 4;
+        const long bufMinSize = 36;
         long bufRead = 0;
         DataBuf buf(bufMinSize);
+        Blob iptcBlob;
+        bool foundPsData = false;
 
         // Read section marker
         int marker = advanceToMarker();
@@ -235,22 +268,32 @@ namespace Exiv2 {
 
         while (marker != sos_ && marker != eoi_ && search > 0) {
             // Read size and signature (ok if this hits EOF)
+            std::memset(buf.pData_, 0x0, buf.size_);
             bufRead = io_->read(buf.pData_, bufMinSize);
             if (io_->error()) throw Error(14);
+            if (bufRead < 2) throw Error(15);
             uint16_t size = getUShort(buf.pData_, bigEndian);
 
+            if (foundPsData && marker != app13_) {
+                // For IPTC, decrement search only after all app13 segments are
+                // loaded, assuming they all appear in sequence. But decode IPTC
+                // data after the loop, in case an app13 is the last segment
+                // before sos or eoi.
+                foundPsData = false;
+                if (--search == 0) break;
+            }
+
             if (marker == app1_ && memcmp(buf.pData_ + 2, exifId_, 6) == 0) {
                 if (size < 8) {
                     rc = 1;
                     break;
                 }
                 // Seek to beginning and read the Exif data
-                io_->seek(8-bufRead, BasicIo::cur);
-                long sizeExifData = size - 8;
-                DataBuf rawExif(sizeExifData);
-                io_->read(rawExif.pData_, sizeExifData);
+                io_->seek(8 - bufRead, BasicIo::cur);
+                DataBuf rawExif(size - 8);
+                io_->read(rawExif.pData_, rawExif.size_);
                 if (io_->error() || io_->eof()) throw Error(14);
-                if (exifData_.load(rawExif.pData_, sizeExifData)) {
+                if (exifData_.load(rawExif.pData_, rawExif.size_)) {
 #ifndef SUPPRESS_WARNINGS
                     std::cerr << "Warning: Failed to decode Exif metadata.\n";
 #endif
@@ -258,6 +301,24 @@ namespace Exiv2 {
                 }
                 --search;
             }
+            else if (marker == app1_ && memcmp(buf.pData_ + 2, xmpId_, 29) == 0) {
+                if (size < 31) {
+                    rc = 6;
+                    break;
+                }
+                // Seek to beginning and read the XMP packet
+                io_->seek(31 - bufRead, BasicIo::cur);
+                DataBuf xmpPacket(size - 31);
+                io_->read(xmpPacket.pData_, xmpPacket.size_);
+                if (io_->error() || io_->eof()) throw Error(14);
+                xmpPacket_.assign(reinterpret_cast<char*>(xmpPacket.pData_), xmpPacket.size_);
+                if (xmpPacket_.size() > 0 && XmpParser::decode(xmpData_, xmpPacket_)) {
+#ifndef SUPPRESS_WARNINGS
+                    std::cerr << "Warning: Failed to decode XMP metadata.\n";
+#endif
+                }
+                --search;
+            }
             else if (   marker == app13_
                      && memcmp(buf.pData_ + 2, Photoshop::ps3Id_, 14) == 0) {
                 if (size < 16) {
@@ -265,26 +326,36 @@ namespace Exiv2 {
                     break;
                 }
                 // Read the rest of the APP13 segment
-                // needed if bufMinSize!=16: io_->seek(16-bufRead, BasicIo::cur);
+                io_->seek(16 - bufRead, BasicIo::cur);
                 DataBuf psData(size - 16);
                 io_->read(psData.pData_, psData.size_);
                 if (io_->error() || io_->eof()) throw Error(14);
                 const byte *record = 0;
                 uint32_t sizeIptc = 0;
                 uint32_t sizeHdr = 0;
-                // Find actual Iptc data within the APP13 segment
-                if (!Photoshop::locateIptcIrb(psData.pData_, psData.size_,
-                                              &record, &sizeHdr, &sizeIptc)) {
-                    if (sizeIptc) {
-                        if (iptcData_.load(record + sizeHdr, sizeIptc)) {
-#ifndef SUPPRESS_WARNINGS
-                            std::cerr << "Warning: Failed to decode IPTC metadata.\n";
+#ifdef DEBUG
+                std::cerr << "Found app13 segment, size = " << size << "\n";
+                //hexdump(std::cerr, psData.pData_, psData.size_);
 #endif
-                            iptcData_.clear();
-                        }
+                // Find actual IPTC data within the APP13 segment
+                const byte* pEnd = psData.pData_ + psData.size_;
+                const byte* pCur = psData.pData_;
+                while (   pCur < pEnd
+                       && 0 == Photoshop::locateIptcIrb(pCur,
+                                                        static_cast<long>(pEnd - pCur),
+                                                        &record,
+                                                        &sizeHdr,
+                                                        &sizeIptc)) {
+                    if (sizeIptc) {
+#ifdef DEBUG
+                        std::cerr << "Found IPTC IRB, size = " << sizeIptc << "\n";
+#endif
+                        append(iptcBlob, record + sizeHdr, sizeIptc);
                     }
+                    pCur = record + sizeHdr + sizeIptc;
+                    pCur += (sizeIptc & 1);
                 }
-                --search;
+                foundPsData = true;
             }
             else if (marker == com_ && comment_.empty())
             {
@@ -292,14 +363,14 @@ namespace Exiv2 {
                     rc = 3;
                     break;
                 }
-                // Jpegs can have multiple comments, but for now only read
+                // JPEGs can have multiple comments, but for now only read
                 // the first one (most jpegs only have one anyway). Comments
                 // are simple single byte ISO-8859-1 strings.
-                io_->seek(2-bufRead, BasicIo::cur);
-                buf.alloc(size-2);
-                io_->read(buf.pData_, size-2);
+                io_->seek(2 - bufRead, BasicIo::cur);
+                DataBuf comment(size - 2);
+                io_->read(comment.pData_, comment.size_);
                 if (io_->error() || io_->eof()) throw Error(14);
-                comment_.assign(reinterpret_cast<char*>(buf.pData_), size-2);
+                comment_.assign(reinterpret_cast<char*>(comment.pData_), comment.size_);
                 while (   comment_.length()
                        && comment_.at(comment_.length()-1) == '\0') {
                     comment_.erase(comment_.length()-1);
@@ -321,6 +392,15 @@ namespace Exiv2 {
                 break;
             }
         } // while there are segments to process
+
+        if (   iptcBlob.size() > 0
+            && iptcData_.load(&iptcBlob[0], static_cast<long>(iptcBlob.size()))) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << "Warning: Failed to decode IPTC metadata.\n";
+#endif
+            iptcData_.clear();
+        }
+
         if (rc != 0) {
 #ifndef SUPPRESS_WARNINGS
             std::cerr << "Warning: JPEG format error, rc = " << rc << "\n";
@@ -353,7 +433,7 @@ namespace Exiv2 {
             throw Error(22);
         }
 
-        const long bufMinSize = 16;
+        const long bufMinSize = 36;
         long bufRead = 0;
         DataBuf buf(bufMinSize);
         const long seek = io_->tell();
@@ -361,6 +441,7 @@ namespace Exiv2 {
         int search = 0;
         int insertPos = 0;
         int skipApp1Exif = -1;
+        int skipApp1Xmp = -1;
         int skipApp13Ps3 = -1;
         int skipCom = -1;
         DataBuf psData;
@@ -375,7 +456,7 @@ namespace Exiv2 {
         // First find segments of interest. Normally app0 is first and we want
         // to insert after it. But if app0 comes after com, app1 and app13 then
         // don't bother.
-        while (marker != sos_ && marker != eoi_ && search < 3) {
+        while (marker != sos_ && marker != eoi_ && search < 4) {
             // Read size and signature (ok if this hits EOF)
             bufRead = io_->read(buf.pData_, bufMinSize);
             if (io_->error()) throw Error(20);
@@ -392,14 +473,23 @@ namespace Exiv2 {
                 ++search;
                 if (io_->seek(size-bufRead, BasicIo::cur)) throw Error(22);
             }
+            else if (marker == app1_ && memcmp(buf.pData_ + 2, xmpId_, 29) == 0) {
+                if (size < 31) throw Error(22);
+                skipApp1Xmp = count;
+                ++search;
+                if (io_->seek(size-bufRead, BasicIo::cur)) throw Error(22);
+            }
             else if (marker == app13_ && memcmp(buf.pData_ + 2, Photoshop::ps3Id_, 14) == 0) {
+#ifdef DEBUG
+                std::cerr << "Found APP13 Photoshop PS3 segment\n";
+#endif
                 if (size < 16) throw Error(22);
                 skipApp13Ps3 = count;
                 ++search;
-                // needed if bufMinSize!=16: io_->seek(16-bufRead, BasicIo::cur);
+                io_->seek(16 - bufRead, BasicIo::cur);
                 psData.alloc(size - 16);
                 // Load PS data now to allow reinsertion at any point
-                io_->read(psData.pData_, psData.size_);
+                io_->read(psData.pData_, size - 16);
                 if (io_->error() || io_->eof()) throw Error(20);
             }
             else if (marker == com_ && skipCom == -1) {
@@ -420,6 +510,8 @@ namespace Exiv2 {
         }
 
         if (exifData_.count() > 0) ++search;
+        if (writeXmpFromPacket() == false && xmpData_.count() > 0) ++search;
+        if (writeXmpFromPacket() == true && xmpPacket_.size() > 0) ++search;
         if (iptcData_.count() > 0) ++search;
         if (!comment_.empty()) ++search;
 
@@ -441,7 +533,7 @@ namespace Exiv2 {
             uint16_t size = getUShort(buf.pData_, bigEndian);
 
             if (insertPos == count) {
-                byte tmpBuf[18];
+                byte tmpBuf[64];
                 if (!comment_.empty()) {
                     // Write COM marker, size of comment, and string
                     tmpBuf[0] = 0xff;
@@ -466,16 +558,39 @@ namespace Exiv2 {
 
                         if (rawExif.size_ + 8 > 0xffff) throw Error(37, "Exif");
                         us2Data(tmpBuf + 2, static_cast<uint16_t>(rawExif.size_ + 8), bigEndian);
-                        memcpy(tmpBuf + 4, exifId_, 6);
+                        std::memcpy(tmpBuf + 4, exifId_, 6);
                         if (outIo.write(tmpBuf, 10) != 10) throw Error(21);
 
                         // Write new Exif data buffer
                         if (   outIo.write(rawExif.pData_, rawExif.size_)
-                               != rawExif.size_) throw Error(21);
+                            != rawExif.size_) throw Error(21);
                         if (outIo.error()) throw Error(21);
                         --search;
                     }
                 }
+                if (writeXmpFromPacket() == false) {
+                    if (XmpParser::encode(xmpPacket_, xmpData_)) {
+#ifndef SUPPRESS_WARNINGS
+                        std::cerr << "Error: Failed to encode XMP metadata.\n";
+#endif
+                    }
+                }
+                if (xmpPacket_.size() > 0) {
+                    // Write APP1 marker, size of APP1 field, XMP id and XMP packet
+                    tmpBuf[0] = 0xff;
+                    tmpBuf[1] = app1_;
+
+                    if (xmpPacket_.size() + 31 > 0xffff) throw Error(37, "XMP");
+                    us2Data(tmpBuf + 2, static_cast<uint16_t>(xmpPacket_.size() + 31), bigEndian);
+                    std::memcpy(tmpBuf + 4, xmpId_, 29);
+                    if (outIo.write(tmpBuf, 33) != 33) throw Error(21);
+
+                    // Write new XMP packet
+                    if (   outIo.write(reinterpret_cast<const byte*>(xmpPacket_.data()), static_cast<long>(xmpPacket_.size()))
+                        != static_cast<long>(xmpPacket_.size())) throw Error(21);
+                    if (outIo.error()) throw Error(21);
+                    --search;
+                }
                 if (psData.size_ > 0 || iptcData_.count() > 0) {
                     // Set the new IPTC IRB, keeps existing IRBs but removes the
                     // IPTC block if there is no new IPTC data to write
@@ -489,7 +604,7 @@ namespace Exiv2 {
 
                         if (newPsData.size_ + 16 > 0xffff) throw Error(37, "IPTC");
                         us2Data(tmpBuf + 2, static_cast<uint16_t>(newPsData.size_ + 16), bigEndian);
-                        memcpy(tmpBuf + 4, Photoshop::ps3Id_, 14);
+                        std::memcpy(tmpBuf + 4, Photoshop::ps3Id_, 14);
                         if (outIo.write(tmpBuf, 18) != 18) throw Error(21);
                         if (outIo.error()) throw Error(21);
 
@@ -506,7 +621,10 @@ namespace Exiv2 {
             if (marker == eoi_) {
                 break;
             }
-            else if (skipApp1Exif==count || skipApp13Ps3==count || skipCom==count) {
+            else if (   skipApp1Exif == count
+                     || skipApp1Xmp  == count
+                     || skipApp13Ps3 == count
+                     || skipCom      == count) {
                 --search;
                 io_->seek(size-bufRead, BasicIo::cur);
             }
@@ -614,7 +732,7 @@ namespace Exiv2 {
         byte tmpBuf[7];
         tmpBuf[0] = 0xff;
         tmpBuf[1] = 0x01;
-        memcpy(tmpBuf + 2, exiv2Id_, 5);
+        std::memcpy(tmpBuf + 2, exiv2Id_, 5);
         if (outIo.write(tmpBuf, 7) != 7) return 4;
         if (outIo.error()) return 4;
         return 0;
