@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      makernote.cpp
-  Version:   $Rev: 1039 $
+  Version:   $Rev: 1366 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   18-Feb-04, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: makernote.cpp 1039 2007-02-02 10:09:31Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: makernote.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
 
 // Define DEBUG_* to output debug information to std::cerr, e.g, by calling
 // make like this: make DEFS=-DDEBUG_MAKERNOTE makernote.o
@@ -44,6 +44,7 @@ EXIV2_RCSID("@(#) $Id: makernote.cpp 1039 2007-02-02 10:09:31Z ahuggel $")
 #include <iomanip>
 #include <iostream>
 #include <cassert>
+#include <cstring>
 
 // *****************************************************************************
 // class member definitions
@@ -74,7 +75,7 @@ namespace Exiv2 {
         : MakerNote(rhs), absShift_(rhs.absShift_), shift_(rhs.shift_),
           start_(rhs.start_), header_(rhs.header_.size_), ifd_(rhs.ifd_)
     {
-        memcpy(header_.pData_, rhs.header_.pData_, header_.size_);
+        std::memcpy(header_.pData_, rhs.header_.pData_, header_.size_);
     }
 
     int IfdMakerNote::read(const byte* buf,
@@ -153,7 +154,7 @@ namespace Exiv2 {
 
     long IfdMakerNote::copyHeader(byte* buf) const
     {
-        if (header_.size_ != 0) memcpy(buf, header_.pData_, header_.size_);
+        if (header_.size_ != 0) std::memcpy(buf, header_.pData_, header_.size_);
         return header_.size_;
     }
 
