@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2006 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,8 +20,9 @@
  */
 /*!
   @file    makernote2.hpp
-  @brief   Makernote related classes
-  @version $Rev: 808 $
+  @brief   Makernote TIFF composite class TiffIfdMakernote and classes for
+           various makernote headers.
+  @version $Rev: 1366 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    11-Apr-06, ahu: created
@@ -48,7 +49,7 @@ namespace Exiv2 {
         const uint16_t canonmn   = 259; //!< Canon makernote
         const uint16_t canoncs   = 260; //!< Canon camera settings
         const uint16_t canonsi   = 261; //!< Canon shot info
-        const uint16_t canoncf   = 262; //!< Canon customer functions
+        const uint16_t canoncf   = 262; //!< Canon custom functions
         const uint16_t nikonmn   = 263; //!< Any Nikon makernote (pseudo group)
         const uint16_t nikon1mn  = 264; //!< Nikon1 makernote
         const uint16_t nikon2mn  = 265; //!< Nikon2 makernote
@@ -63,6 +64,9 @@ namespace Exiv2 {
         const uint16_t minocsn   = 274; //!< Minolta camera settings (new)
         const uint16_t minocs5   = 275; //!< Minolta camera settings (D5)
         const uint16_t minocs7   = 276; //!< Minolta camera settings (D7)
+        const uint16_t canonpi   = 277; //!< Canon picture info
+        const uint16_t canonpa   = 278; //!< Canon panorama
+        const uint16_t pentaxmn  = 279; //!< Pentax makernote
     }
 
 // *****************************************************************************
@@ -160,12 +164,10 @@ namespace Exiv2 {
         virtual ByteOrder byteOrder() const { return invalidByteOrder; }
         /*!
           @brief Return the base offset for the makernote IFD entries relative
-                 to the start of the TIFF header.
-
-          @param mnOffset Offset to the makernote from the start of the
-                 TIFF header.
+                 to the start of the TIFF header. \em mnOffset is the offset
+                 to the makernote from the start of the TIFF header.
          */
-        virtual uint32_t baseOffset(uint32_t mnOffset) const { return 0; }
+        virtual uint32_t baseOffset(uint32_t /*mnOffset*/) const { return 0; }
         //@}
 
     }; // class MnHeader
@@ -401,6 +403,35 @@ namespace Exiv2 {
 
     }; // class PanasonicMnHeader
 
+    //! Header of an Pentax Makernote
+    class PentaxMnHeader : public MnHeader {
+    public:
+        //! @name Creators
+        //@{
+        //! Default constructor
+        PentaxMnHeader();
+        //! Virtual destructor.
+        virtual ~PentaxMnHeader() {}
+        //@}
+        //! @name Manipulators
+        //@{
+        virtual bool read(const byte* pData,
+                          uint32_t    size,
+                          ByteOrder   byteOrder);
+        //@}
+        //! @name Accessors
+        //@{
+        virtual uint32_t size()      const { return header_.size_; }
+        virtual uint32_t ifdOffset() const { return size_; }
+        //@}
+
+    private:
+        DataBuf header_;                //!< Data buffer for the makernote header
+        static const byte signature_[]; //!< Pentax makernote header signature
+        static const uint32_t size_;    //!< Size of the signature
+
+    }; // class PentaxMnHeader
+
     //! Header of a Sigma Makernote
     class SigmaMnHeader : public MnHeader {
     public:
@@ -516,6 +547,14 @@ namespace Exiv2 {
                                   uint32_t    size,
                                   ByteOrder   byteOrder);
 
+    //! Function to create an Pentax makernote
+    TiffComponent* newPentaxMn(uint16_t    tag,
+                                uint16_t    group,
+                                uint16_t    mnGroup,
+                                const byte* pData,
+                                uint32_t    size,
+                                ByteOrder   byteOrder);
+
     //! Function to create a Sigma makernote
     TiffComponent* newSigmaMn(uint16_t    tag,
                               uint16_t    group,
