@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2006-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,8 +20,8 @@
  */
 /*
   File:      minoltamn.cpp
-  Version:   $Rev: 1039 $
-  Author(s): Gilles Caulier (gc) <caulier.gilles@kdemail.net>
+  Version:   $Rev: 1366 $
+  Author(s): Gilles Caulier (cgilles) <caulier dot gilles at gmail dot com>
              Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   06-May-06, gc: submitted
   Credits:   See header file.
@@ -29,7 +29,7 @@
 
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: minoltamn.cpp 1039 2007-02-02 10:09:31Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: minoltamn.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -44,6 +44,7 @@ EXIV2_RCSID("@(#) $Id: minoltamn.cpp 1039 2007-02-02 10:09:31Z ahuggel $")
 #include <sstream>
 #include <iomanip>
 #include <cassert>
+#include <cstring>
 
 // *****************************************************************************
 // class member definitions
@@ -100,7 +101,6 @@ namespace Exiv2 {
        25720/25721, 25790/25791, 25960/25961, 25980/25981, 26150/26151
        - No need to i18n these string.
     */
-
     extern const TagDetails minoltaLensID[] = {
         { 0,     "AF28-85mm F3.5-4.5"                                                         },
         { 1,     "AF80-200mm F2.8G"                                                           },
@@ -126,7 +126,7 @@ namespace Exiv2 {
         { 22,    "AF35-80mm F4-5.6"                                                           },
         { 23,    "AF200mm F4G APO Macro"                                                      },
         { 24,    "AF24-105mm F3.5-4.5(D) | SIGMA 18-50mm F2.8 | Sigma DC 18-125mm F4-5,6 D"   },
-        { 25,    "AF100-300mm F4.5-5.6 APO(D)"                                                },
+        { 25,    "AF100-300mm F4.5-5.6 APO(D) | SIGMA 30mm F1.4"                              },
         { 27,    "AF85mm F1.4G(D)"                                                            },
         { 28,    "AF100mm F2.8 Macro(D)"                                                      },
         { 29,    "AF75-300mm F4.5-5.6(D)"                                                     },
@@ -141,22 +141,31 @@ namespace Exiv2 {
         { 40,    "AFDT18-70mm F3.5-5.6(D)"                                                    },
         { 41,    "AFDT11-18mm F4.5-5.6(D)"                                                    },
         { 42,    "AFDT18-200mm F3.5-6.3(D)"                                                   },
+        { 45,    "CZ Planar T* AF85mm F1.4"                                                   },
+        { 46,    "CZ Vario-Sonnar T* AFDT16-80mm F3.5-4.5"                                    },
+        { 47,    "CZ Sonnar T* 135mm F1.8"                                                    },
         { 128,   "TAMRON 18-200, 28-300 | 80-300mm F3.5-6.3"                                  },
+        { 193,   "AF 1.4x APO II"                                                             },
         { 25500, "AF50mm F1.7"                                                                },
         { 25501, "AF50mm F1.7"                                                                },
         { 25510, "AF35-70mm F1.4"                                                             },
+        { 25511, "AF35-70mm F4"                                                               },
         { 25520, "AF28-85mm F3.5-4.5"                                                         },
         { 25521, "TOKINA 19-35mm F3.5-4.5 | TOKINA 28-70mm F2.8 AT-X"                         },
         { 25530, "AF28-135mm F4-4.5"                                                          },
+        { 25531, "AF28-135mm F4-4.5"                                                          },
         { 25540, "AF35-105mm F3.5-4.5"                                                        },
         { 25541, "AF35-105mm F3.5-4.5"                                                        },
         { 25550, "AF70-210mm F4"                                                              },
         { 25551, "AF70-210mm F4 Macro | SIGMA 70-210mm F4-5.6 APO"                            },
         { 25560, "AF135mm F2.8"                                                               },
+        { 25561, "AF135mm F2.8"                                                               },
         { 25570, "AF28mm F2.8"                                                                },
+        { 25571, "AF28mm F2.8"                                                                },
         { 25580, "AF24-50mm F4"                                                               },
         { 25581, "AF24-50mm F4"                                                               },
         { 25600, "AF100-200mm F4.5"                                                           },
+        { 25601, "AF100-200mm F4.5"                                                           },
         { 25610, "AF75-300mm F4.5-5.6"                                                        },
         { 25611, "SIGMA 70-300mm F4-5.6 | SIGMA 300mm F4 APO Macro"                           },
         { 25620, "AF50mm F1.4"                                                                },
@@ -182,6 +191,7 @@ namespace Exiv2 {
         { 25890, "AF80-200mm F2.8 APO"                                                        },
         { 25891, "TOKINA 80-200mm F2.8"                                                       },
         { 25910, "AF35mm F1.4G"                                                               },
+        { 25911, "AF35mm F1.4G"                                                               },
         { 25920, "AF85mm F1.4G"                                                               },
         { 25921, "AF85mm F1.4G(D)"                                                            },
         { 25930, "AF200mm F2.8 APO"                                                           },
@@ -190,6 +200,7 @@ namespace Exiv2 {
         { 25960, "AF28mm F2"                                                                  },
         { 25961, "AF28mm F2"                                                                  },
         { 25970, "AF35mm F2"                                                                  },
+        { 25971, "AF35mm F2"                                                                  },
         { 25980, "AF100mm F2"                                                                 },
         { 25981, "AF100mm F2"                                                                 },
         { 26040, "AF80-200mm F4.5-5.6"                                                        },
@@ -197,6 +208,7 @@ namespace Exiv2 {
         { 26060, "AF100-300mm F4.5-5.6"                                                       },
         { 26061, "AF100-300mm F4.5-5.6(D) | SIGMA 105mm F2.8 Macro EX-DG"                     },
         { 26070, "AF35-80mm F4-5.6"                                                           },
+        { 26071, "AF35-80mm F4-5.6"                                                           },
         { 26080, "AF300mm F2.8G APO High Speed"                                               },
         { 26081, "AF300mm F2.8G"                                                              },
         { 26090, "AF600mm F4G APO High Speed"                                                 },
@@ -207,6 +219,7 @@ namespace Exiv2 {
         { 26150, "AF28-105mm F3.5-4.5 Xi"                                                     },
         { 26151, "AF28-105mm F3.5-4.5 Xi"                                                     },
         { 26160, "AF35-200mm F4.5-5.6 Xi"                                                     },
+        { 26161, "AF35-200mm F4.5-5.6 Xi"                                                     },
         { 26180, "AF28-80mm F4-5.6 Xi"                                                        },
         { 26190, "AF80-200mm F4.5-5.6 Xi"                                                     },
         { 26210, "AF100-300mm F4.5-5.6 Xi"                                                    },
@@ -1422,7 +1435,7 @@ namespace Exiv2 {
     long MinoltaMakerNote::assemble(Entry& e, IfdId ifdId, uint16_t tag, ByteOrder /*byteOrder*/) const
     {
         DataBuf buf(1024);
-        memset(buf.pData_, 0x0, 1024);
+        std::memset(buf.pData_, 0x0, 1024);
         uint16_t len = 0;
         Entries::const_iterator end = entries_.end();
         for (Entries::const_iterator i = entries_.begin(); i != end; ++i) {
@@ -1430,7 +1443,7 @@ namespace Exiv2 {
                 uint16_t pos = i->tag() * 2;
                 uint16_t size = pos + static_cast<uint16_t>(i->size());
                 assert(size <= 1024);
-                memcpy(buf.pData_ + pos, i->data(), i->size());
+                std::memcpy(buf.pData_ + pos, i->data(), i->size());
                 if (len < size) len = size;
             }
         }
@@ -1447,7 +1460,7 @@ namespace Exiv2 {
     long MinoltaMakerNote::assembleStd(Entry& e, IfdId ifdId, uint32_t tag, ByteOrder /*byteOrder*/) const
     {
         DataBuf buf(1024);
-        memset(buf.pData_, 0x0, 1024);
+        std::memset(buf.pData_, 0x0, 1024);
         uint32_t len = 0;
         Entries::const_iterator end = entries_.end();
         for (Entries::const_iterator i = entries_.begin(); i != end; ++i) {
@@ -1455,7 +1468,7 @@ namespace Exiv2 {
                 uint32_t pos = i->tag() * 4;
                 uint32_t size = pos + static_cast<uint32_t>(i->size());
                 assert(size <= 1024);
-                memcpy(buf.pData_ + pos, i->data(), i->size());
+                std::memcpy(buf.pData_ + pos, i->data(), i->size());
                 if (len < size) len = size;
             }
         }
