@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,7 +20,7 @@
  */
 /*
   File:      panasonicmn.cpp
-  Version:   $Rev: 1039 $
+  Version:   $Rev: 1366 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Gilles Caulier (gc) <caulier.gilles@kdemail.net>
   History:   11-Jun-04, ahu: created
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: panasonicmn.cpp 1039 2007-02-02 10:09:31Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: panasonicmn.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -43,6 +43,7 @@ EXIV2_RCSID("@(#) $Id: panasonicmn.cpp 1039 2007-02-02 10:09:31Z ahuggel $")
 #include <sstream>
 #include <iomanip>
 #include <cassert>
+#include <cstring>
 
 // *****************************************************************************
 // class member definitions
@@ -306,7 +307,7 @@ namespace Exiv2 {
         if (len < 12) return 1;
 
         header_.alloc(12);
-        memcpy(header_.pData_, buf, header_.size_);
+        std::memcpy(header_.pData_, buf, header_.size_);
         // Adjust the offset of the IFD for the prefix
         start_ = 12;
         return 0;
