@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,7 +20,7 @@
  */
 /*
   File:      tags.cpp
-  Version:   $Rev: 1113 $
+  Version:   $Rev: 1366 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Gilles Caulier (gc) <caulier.gilles@kdemail.net>
   History:   15-Jan-04, ahu: created
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: tags.cpp 1113 2007-06-03 08:32:16Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: tags.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -89,6 +89,7 @@ namespace Exiv2 {
         IfdInfo(nikon3IfdId, "Makernote", "Nikon3"),
         IfdInfo(olympusIfdId, "Makernote", "Olympus"),
         IfdInfo(panasonicIfdId, "Makernote", "Panasonic"),
+        IfdInfo(pentaxIfdId, "Makernote", "Pentax"),
         IfdInfo(sigmaIfdId, "Makernote", "Sigma"),
         IfdInfo(sonyIfdId, "Makernote", "Sony"),
         IfdInfo(lastIfdId, "(Last IFD info)", "(Last IFD item)")
@@ -169,6 +170,7 @@ namespace Exiv2 {
         {     9, N_("JBIG B&W")                 },
         {    10, N_("JBIG Color")               },
         { 32766, N_("Next 2-bits RLE")          },
+        { 32769, N_("Epson ERF Compressed")     },
         { 32771, N_("CCITT RLE 1-word")         },
         { 32773, N_("PackBits (Macintosh RLE)") },
         { 32809, N_("Thunderscan RLE")          },
@@ -184,7 +186,9 @@ namespace Exiv2 {
         { 34676, N_("SGI Log Luminance RLE")    },
         { 34677, N_("SGI Log 24-bits packed")   },
         { 34712, N_("Leadtools JPEG 2000")      },
-        { 34713, N_("Nikon NEF Compressed")     }
+        { 34713, N_("Nikon NEF Compressed")     },
+        { 65000, N_("Kodak DCR Compressed")     },
+        { 65535, N_("Pentax PEF Compressed")    }
     };
 
     //! PhotometricInterpretation, tag 0x0106
@@ -498,6 +502,9 @@ namespace Exiv2 {
         TagInfo(0x9c9f, "XPSubject", N_("Windows Subject"),
                 N_("Subject tag used by Windows, encoded in UCS2"),
                 ifd0Id, otherTags, unsignedByte, printUcs2), // Windows Tag
+	TagInfo(0xc4a5, "PrintImageMatching", N_("Print Image Matching"),
+		N_("Print Image Matching, descriptiont needed."),
+		ifd0Id, otherTags, undefined, printValue),
         // End of list marker
         TagInfo(0xffff, "(UnknownIfdTag)", N_("Unknown IFD tag"),
                 N_("Unknown IFD tag"),
@@ -565,21 +572,26 @@ namespace Exiv2 {
         { 0x01, N_("Fired")                                                         },
         { 0x05, N_("Fired, strobe return light not detected")                       },
         { 0x07, N_("Fired, strobe return light detected")                           },
+        { 0x08, N_("Yes, did not fire")                                             },
         { 0x09, N_("Yes, compulsory")                                               },
         { 0x0d, N_("Yes, compulsory, return light not detected")                    },
         { 0x0f, N_("Yes, compulsory, return light detected")                        },
         { 0x10, N_("No, compulsory")                                                },
+        { 0x14, N_("No, did not fire, return not detected")                         },
         { 0x18, N_("No, auto")                                                      },
         { 0x19, N_("Yes, auto")                                                     },
         { 0x1d, N_("Yes, auto, return light not detected")                          },
         { 0x1f, N_("Yes, auto, return light detected")                              },
         { 0x20, N_("No flash function")                                             },
+        { 0x20, N_("No, no flash function")                                         },
         { 0x41, N_("Yes, red-eye reduction")                                        },
         { 0x45, N_("Yes, red-eye reduction, return light not detected")             },
         { 0x47, N_("Yes, red-eye reduction, return light detected")                 },
         { 0x49, N_("Yes, compulsory, red-eye reduction")                            },
         { 0x4d, N_("Yes, compulsory, red-eye reduction, return light not detected") },
         { 0x4f, N_("Yes, compulsory, red-eye reduction, return light detected")     },
+        { 0x50, N_("No, red-eye reduction")                                         },
+        { 0x58, N_("No, auto, red-eye reduction")                                   },
         { 0x59, N_("Yes, auto, red-eye reduction")                                  },
         { 0x5d, N_("Yes, auto, red-eye reduction, return light not detected")       },
         { 0x5f, N_("Yes, auto, red-eye reduction, return light detected")           }
@@ -1617,7 +1629,8 @@ namespace Exiv2 {
         int32_t denominator;
         char c;
         is >> nominator >> c >> denominator;
-        if (is && c == '/') r = std::make_pair(nominator, denominator);
+        if (c != '/') is.setstate(std::ios::failbit);
+        if (is) r = std::make_pair(nominator, denominator);
         return is;
     }
 
@@ -1630,9 +1643,10 @@ namespace Exiv2 {
     {
         uint32_t nominator;
         uint32_t denominator;
-        char c;
+        char c('\0');
         is >> nominator >> c >> denominator;
-        if (is && c == '/') r = std::make_pair(nominator, denominator);
+        if (c != '/') is.setstate(std::ios::failbit);
+        if (is) r = std::make_pair(nominator, denominator);
         return is;
     }
 
