@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    tags.hpp
   @brief   Exif tag and type information
-  @version $Rev: 986 $
+  @version $Rev: 1366 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    15-Jan-04, ahu: created<BR>
@@ -118,7 +118,7 @@ namespace Exiv2 {
      */
     struct TagDetails {
         long val_;                              //!< Tag value
-        char* label_;                           //!< Translation of the tag value
+        const char* label_;                     //!< Translation of the tag value
 
         //! Comparison operator for use with the find template
         bool operator==(long key) const { return val_ == key; }
@@ -130,7 +130,7 @@ namespace Exiv2 {
      */
     struct TagDetailsBitmask {
         uint32_t mask_;                         //!< Bitmask value
-        char* label_;                           //!< Description of the tag value
+        const char* label_;                     //!< Description of the tag value
     }; // struct TagDetailsBitmask
 
     /*!
@@ -142,7 +142,7 @@ namespace Exiv2 {
     {
         const TagDetails* td = find(array, value.toLong());
         if (td) {
-            os << td->label_;
+            os << exvGettext(td->label_);
         }
         else {
             os << "(" << value << ")";
@@ -168,10 +168,10 @@ namespace Exiv2 {
 
             if (val & td->mask_) {
                 if (sep) {
-                    os << ", " << td->label_;
+                    os << ", " << exvGettext(td->label_);
                 }
                 else {
-                    os << td->label_;
+                    os << exvGettext(td->label_);
                     sep = true;
                 }
             }
@@ -442,6 +442,8 @@ namespace Exiv2 {
     std::ostream& printFloat(std::ostream& os, const Value& value);
     //! Print a longitude or latitude value
     std::ostream& printDegrees(std::ostream& os, const Value& value);
+    //! Print function converting from UCS-2LE to UTF-8
+    std::ostream& printUcs2(std::ostream& os, const Value& value);
 
     //! Print GPS altitude
     std::ostream& print0x0006(std::ostream& os, const Value& value);
