@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2006 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    tiffimage.hpp
   @brief   Class TiffImage
-  @version $Rev: 797 $
+  @version $Rev: 1366 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    15-Mar-06, ahu: created
@@ -31,9 +31,8 @@
 
 // *****************************************************************************
 // included header files
-#include "exif.hpp"
-#include "iptc.hpp"
 #include "image.hpp"
+#include "basicio.hpp"
 
 // + standard includes
 #include <string>
@@ -51,20 +50,10 @@ namespace Exiv2 {
     }
 
     /*!
-      @brief Class to access raw TIFF images. Exif metadata and a comment
-             are supported directly, IPTC is read from the Exif data, if present.
+      @brief Class to access TIFF images. Exif metadata is
+          supported directly, IPTC is read from the Exif data, if present.
      */
     class TiffImage : public Image {
-        friend bool isTiffType(BasicIo& iIo, bool advance);
-
-        //! @name NOT Implemented
-        //@{
-        //! Copy constructor
-        TiffImage(const TiffImage& rhs);
-        //! Assignment operator
-        TiffImage& operator=(const TiffImage& rhs);
-        //@}
-
     public:
         //! @name Creators
         //@{
@@ -84,136 +73,133 @@ namespace Exiv2 {
               or if a new file should be created (true).
          */
         TiffImage(BasicIo::AutoPtr io, bool create);
-        //! Destructor
-        ~TiffImage() {}
         //@}
 
         //! @name Manipulators
         //@{
-        void            readMetadata();
+        void readMetadata();
         /*!
           @brief Todo: Write metadata back to the image. This method is not
-                 yet implemented.
+              yet implemented. Calling it will throw an Error(31).
          */
-        void            writeMetadata();
-        void            setExifData(const ExifData& exifData);
-        void            clearExifData();
-        void            setIptcData(const IptcData& iptcData);
-        void            clearIptcData();
-        void            setComment(const std::string& comment);
-        void            clearComment();
-        void            setMetadata(const Image& image);
-        void            clearMetadata();
-        ExifData&       exifData()       { return exifData_; }
-        IptcData&       iptcData()       { return iptcData_; }
+        void writeMetadata();
+        /*!
+          @brief Todo: Not supported yet, requires writeMetadata(). Calling 
+              this function will throw an Error(32).
+         */
+        void setExifData(const ExifData& exifData);
+        /*!
+          @brief Todo: Not supported yet, requires writeMetadata(). Calling 
+              this function will throw an Error(32).
+         */
+        void setIptcData(const IptcData& iptcData);
+        /*!
+          @brief Not supported. TIFF format does not contain a comment.
+              Calling this function will throw an Error(32).
+         */
+        void setComment(const std::string& comment);
         //@}
 
         //! @name Accessors
         //@{
-        bool            good()     const;
-        const ExifData& exifData() const { return exifData_; }
-        const IptcData& iptcData() const { return iptcData_; }
-        std::string     comment()  const { return comment_; }
-        BasicIo&        io()       const { return *io_; }
+        std::string mimeType() const { return "image/tiff"; }
         //@}
 
     private:
-        //! @name Accessors
+        //! @name NOT Implemented
         //@{
-        /*!
-          @brief Determine if the content of the BasicIo instance is a TIFF image.
-
-          The advance flag determines if the read position in the stream is
-          moved (see below). This applies only if the type matches and the
-          function returns true. If the type does not match, the stream
-          position is not changed. However, if reading from the stream fails,
-          the stream position is undefined. Consult the stream state to obtain
-          more information in this case.
-
-          @param iIo BasicIo instance to read from.
-          @param advance Flag indicating whether the position of the io
-              should be advanced by the number of characters read to
-              analyse the data (true) or left at its original
-              position (false). This applies only if the type matches.
-          @return  true  if the data matches the type of this class;<BR>
-                   false if the data does not match
-         */
-        bool isThisType(BasicIo& iIo, bool advance) const;
-        /*!
-          @brief Todo: Write TIFF header. Not implemented yet.
-         */
-        int writeHeader(BasicIo& oIo) const;
+        //! Copy constructor
+        TiffImage(const TiffImage& rhs);
+        //! Assignment operator
+        TiffImage& operator=(const TiffImage& rhs);
         //@}
 
-        // DATA
-        BasicIo::AutoPtr  io_;                  //!< Image data io pointer
-        ExifData          exifData_;            //!< Exif data container
-        IptcData          iptcData_;            //!< IPTC data container
-        std::string       comment_;             //!< User comment
-
     }; // class TiffImage
 
     /*!
-      @brief This class models a TIFF header structure.
+      @brief Abstract base class defining the interface of an image header.
+             Used internally by classes for TIFF-based images.  Default
+             implementation is for the regular TIFF header.
      */
-    class TiffHeade2 {
+    class TiffHeaderBase {
     public:
-        //! @name Creators
+        //! @name Constructors
         //@{
-        //! Default constructor
-        TiffHeade2()
-            : byteOrder_ (littleEndian),
-              offset_    (0x00000008)
-            {}
+        //! Constructor taking \em tag, \em size and default \em byteOrder and \em offset.
+        TiffHeaderBase(uint16_t  tag,
+                       uint32_t  size,
+                       ByteOrder byteOrder,
+                       uint32_t  offset);
+        //! Virtual destructor.
+        virtual ~TiffHeaderBase() =0;
         //@}
 
         //! @name Manipulators
         //@{
         /*!
-          @brief Read the TIFF header from a data buffer. Return false if the
-                 data buffer does not contain a TIFF header, else true.
+          @brief Read the image header from a data buffer. Return false if the
+                 data buffer does not contain an image header of the expected
+                 format, else true.
 
           @param pData Pointer to the data buffer.
           @param size  Number of bytes in the data buffer.
          */
-        bool read(const byte* pData, uint32_t size);
+        virtual bool read(const byte* pData, uint32_t size);
+        //! Set the byte order.
+        virtual void setByteOrder(ByteOrder byteOrder);
+        //! Set the offset to the start of the root directory.
+        virtual void setOffset(uint32_t offset);
         //@}
 
         //! @name Accessors
         //@{
         /*!
-          @brief Write the TIFF header to the binary image \em blob.
+          @brief Write the image header to the binary image \em blob.
                  This method appends to the blob.
 
           @param blob Binary image to add to.
 
           @throw Error If the header cannot be written.
          */
-        void write(Blob& blob) const;
+        virtual void write(Blob& blob) const;
         /*!
-          @brief Print debug info for the TIFF header to \em os.
+          @brief Print debug info for the image header to \em os.
 
           @param os Output stream to write to.
           @param prefix Prefix to be written before each line of output.
          */
-        void print(std::ostream& os, const std::string& prefix ="") const;
+        virtual void print(std::ostream& os, const std::string& prefix ="") const;
         //! Return the byte order (little or big endian).
-        ByteOrder byteOrder() const { return byteOrder_; }
-        //! Return the offset to the start of the root directory
-        uint32_t ifdOffset() const { return offset_; }
-        //! Return the size (in bytes) of the TIFF header
-        uint32_t size() const { return 8; }
+        virtual ByteOrder byteOrder() const;
+        //! Return the offset to the start of the root directory.
+        virtual uint32_t offset() const;
+        //! Return the size (in bytes) of the image header.
+        virtual uint32_t size() const;
         //! Return the tag value (magic number) which identifies the buffer as TIFF data
-        uint16_t tag() const { return tag_; }
+        virtual uint16_t tag() const;
         //@}
 
     private:
         // DATA
-        ByteOrder             byteOrder_; //!< Applicable byte order
-        uint32_t              offset_;    //!< Offset to the start of the root dir
+        const uint16_t tag_;       //!< Tag to identify the buffer as TIFF data
+        const uint32_t size_;      //!< Size of the header
+        ByteOrder      byteOrder_; //!< Applicable byte order
+        uint32_t       offset_;    //!< Offset to the start of the root dir
 
-        static const uint16_t tag_;       //!< 42, identifies the buffer as TIFF data
+    }; // class TiffHeaderBase
 
+    /*!
+      @brief Standard TIFF header structure.
+     */
+    class TiffHeade2 : public TiffHeaderBase {
+    public:
+        //! @name Creators
+        //@{
+        //! Default constructor
+        TiffHeade2();
+        //! Destructor
+        ~TiffHeade2();
+        //@}
     }; // class TiffHeade2
 
 // *****************************************************************************
