@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2006 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    tiffparser.hpp
   @brief   Class TiffParser to parse TIFF data.
-  @version $Rev: 837 $
+  @version $Rev: 1366 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    15-Mar-06, ahu: created
@@ -86,12 +86,15 @@ namespace Exiv2 {
           @param size      Length of the data buffer.
           @param createFct Factory function to create new TIFF components.
           @param findDecoderFct Function to access special decoding info.
+          @param pHeader   Optional pointer to a TIFF header. If not provided,
+                           a standard TIFF header is used.
         */
         static void decode(      Image*             pImage,
                            const byte*              pData,
                                  uint32_t           size,
                                  TiffCompFactoryFct createFct,
-                                 FindDecoderFct     findDecoderFct);
+                                 FindDecoderFct     findDecoderFct,
+                                 TiffHeaderBase*    pHeader =0);
     }; // class TiffParser
 
     /*!
@@ -104,9 +107,9 @@ namespace Exiv2 {
     class TiffDecoder {
     public:
         /*!
-          @brief Find the decoder function for a key. 
+          @brief Find the decoder function for a key.
 
-          If the returned pointer is 0, the tag should not be decoded, 
+          If the returned pointer is 0, the tag should not be decoded,
           else the decoder function should be used.
 
           @param make Camera make
@@ -115,7 +118,7 @@ namespace Exiv2 {
 
           @return Pointer to the decoder function
          */
-        static const DecoderFct findDecoder(const std::string& make, 
+        static const DecoderFct findDecoder(const std::string& make,
                                                   uint32_t     extendedTag,
                                                   uint16_t     group);
 
