@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      types.cpp
-  Version:   $Rev: 1039 $
+  Version:   $Rev: 1366 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: types.cpp 1039 2007-02-02 10:09:31Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: types.cpp 1366 2008-01-09 16:44:09Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -44,6 +44,7 @@ EXIV2_RCSID("@(#) $Id: types.cpp 1039 2007-02-02 10:09:31Z ahuggel $")
 #include <ctime>
 #include <cstdio>
 #include <cassert>
+#include <cstring>
 
 // *****************************************************************************
 // class member definitions
@@ -62,16 +63,21 @@ namespace Exiv2 {
         TypeInfoTable(unsignedShort,    "Short",       2),
         TypeInfoTable(unsignedLong,     "Long",        4),
         TypeInfoTable(unsignedRational, "Rational",    8),
-        TypeInfoTable(invalid6,         "Invalid(6)",  1),
+        TypeInfoTable(signedByte,       "SByte",       1),
         TypeInfoTable(undefined,        "Undefined",   1),
         TypeInfoTable(signedShort,      "SShort",      2),
         TypeInfoTable(signedLong,       "SLong",       4),
         TypeInfoTable(signedRational,   "SRational",   8),
         TypeInfoTable(string,           "String",      1),
         TypeInfoTable(date,             "Date",        8),
-        TypeInfoTable(time,             "Time",        11),
+        TypeInfoTable(time,             "Time",       11),
         TypeInfoTable(comment,          "Comment",     1),
         TypeInfoTable(directory,        "Directory",   1),
+        TypeInfoTable(xmpText,          "XmpText",     1),
+        TypeInfoTable(xmpAlt,           "XmpAlt",      1),
+        TypeInfoTable(xmpBag,           "XmpBag",      1),
+        TypeInfoTable(xmpSeq,           "XmpSeq",      1),
+        TypeInfoTable(langAlt,          "LangAlt",     1),
         // End of list marker
         TypeInfoTable(lastTypeId,       "(Unknown)",   0)
     };
@@ -106,7 +112,7 @@ namespace Exiv2 {
     {
         if (size > 0) {
             pData_ = new byte[size];
-            memcpy(pData_, pData, size);
+            std::memcpy(pData_, pData, size);
             size_ = size;
         }
     }
@@ -321,8 +327,8 @@ namespace Exiv2 {
         assert(tm != 0);
         int rc = 1;
         int year, mon, mday, hour, min, sec;
-        int scanned = sscanf(buf, "%4d:%2d:%2d %2d:%2d:%2d",
-                             &year, &mon, &mday, &hour, &min, &sec);
+        int scanned = std::sscanf(buf, "%4d:%2d:%2d %2d:%2d:%2d",
+                                  &year, &mon, &mday, &hour, &min, &sec);
         if (scanned == 6) {
             tm->tm_year = year - 1900;
             tm->tm_mon  = mon - 1;
