@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2007 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2008 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    types.hpp
   @brief   Type definitions for %Exiv2 and related functionality
-  @version $Rev: 1071 $
+  @version $Rev: 1366 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    09-Jan-04, ahu: created<BR>
@@ -90,18 +90,19 @@ namespace Exiv2 {
     enum ByteOrder { invalidByteOrder, littleEndian, bigEndian };
 
     //! An identifier for each type of metadata
-    enum MetadataId { mdExif=1, mdIptc=2, mdComment=4 };
+    enum MetadataId { mdExif=1, mdIptc=2, mdComment=4, mdXmp=8 };
 
     //! An identifier for each mode of metadata support
     enum AccessMode { amNone=0, amRead=1, amWrite=2, amReadWrite=3 };
 
     //! Type identifiers for IFD format types
     enum TypeId { invalidTypeId, unsignedByte, asciiString, unsignedShort,
-                  unsignedLong, unsignedRational, invalid6, undefined,
+                  unsignedLong, unsignedRational, signedByte, undefined,
                   signedShort, signedLong, signedRational,
                   string, date, time,
                   comment,
                   directory,
+                  xmpText, xmpAlt, xmpBag, xmpSeq, langAlt,
                   lastTypeId };
 
     // Todo: decentralize IfdId, so that new ids can be defined elsewhere
@@ -114,7 +115,7 @@ namespace Exiv2 {
                  minoltaIfdId, minoltaCs5DIfdId, minoltaCs7DIfdId,
                  minoltaCsOldIfdId, minoltaCsNewIfdId,
                  nikon1IfdId, nikon2IfdId, nikon3IfdId,
-                 olympusIfdId, panasonicIfdId, sigmaIfdId, sonyIfdId,
+                 olympusIfdId, panasonicIfdId, pentaxIfdId, sigmaIfdId, sonyIfdId,
                  lastIfdId };
 
     //! Type to identify where the data is stored in a directory
@@ -388,6 +389,16 @@ namespace Exiv2 {
         return os.str();
     }
 
+    //! Utility function to convert a string to a value of type T.
+    template<typename T>
+    T stringTo(const std::string& s, bool& ok)
+    {
+        std::istringstream is(s);
+        T tmp;
+        ok = is >> tmp ? true : false;
+        return tmp;
+    }
+
     /*!
       @brief Return the greatest common denominator of n and m.
              (implementation from Boost rational.hpp)
